@@ -16,7 +16,7 @@ SQLAlchemy models for idds relational data
 import datetime
 from enum import Enum
 
-from sqlalchemy import BigInteger, Boolean, Column, DateTime, Integer, String, event, DDL
+from sqlalchemy import BigInteger, Boolean, Column, DateTime, Integer, String, event, DDL, Interval
 from sqlalchemy.ext.compiler import compiles
 # from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import object_mapper
@@ -30,7 +30,9 @@ from idds.common.constants import (RequestType, RequestStatus, RequestLocking,
                                    CollectionRelationType, ContentType, ContentRelationType,
                                    ContentStatus, ContentLocking, GranularityType,
                                    MessageType, MessageStatus, MessageLocking,
-                                   MessageSource, MessageDestination)
+                                   MessageSource, MessageDestination,
+                                   CommandType, CommandStatus, CommandLocking,
+                                   CommandLocation)
 from idds.common.utils import date_to_str
 from idds.orm.base.enum import EnumSymbol
 from idds.orm.base.types import JSON, JSONString, EnumWithValue
@@ -142,12 +144,19 @@ class Request(BASE, ModelBase):
     priority = Column(Integer())
     status = Column(EnumWithValue(RequestStatus))
     substatus = Column(EnumWithValue(RequestStatus), default=0)
+    oldstatus = Column(EnumWithValue(RequestStatus), default=0)
     locking = Column(EnumWithValue(RequestLocking))
     created_at = Column("created_at", DateTime, default=datetime.datetime.utcnow)
     updated_at = Column("updated_at", DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)
     next_poll_at = Column("next_poll_at", DateTime, default=datetime.datetime.utcnow)
     accessed_at = Column("accessed_at", DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)
     expired_at = Column("expired_at", DateTime)
+    new_retries = Column(Integer(), default=0)
+    update_retries = Column(Integer(), default=0)
+    max_new_retries = Column(Integer(), default=3)
+    max_update_retries = Column(Integer(), default=0)
+    new_poll_period = Column(Interval(), default=datetime.timedelta(seconds=1))
+    update_poll_period = Column(Interval(), default=datetime.timedelta(seconds=10))
     errors = Column(JSONString(1024))
     _request_metadata = Column('request_metadata', JSON())
     _processing_metadata = Column('processing_metadata', JSON())
@@ -258,6 +267,7 @@ class Transform(BASE, ModelBase):
     safe2get_output_from_input = Column(Integer())
     status = Column(EnumWithValue(TransformStatus))
     substatus = Column(EnumWithValue(TransformStatus), default=0)
+    oldstatus = Column(EnumWithValue(TransformStatus), default=0)
     locking = Column(EnumWithValue(TransformLocking))
     retries = Column(Integer(), default=0)
     created_at = Column("created_at", DateTime, default=datetime.datetime.utcnow)
@@ -266,6 +276,13 @@ class Transform(BASE, ModelBase):
     started_at = Column("started_at", DateTime)
     finished_at = Column("finished_at", DateTime)
     expired_at = Column("expired_at", DateTime)
+    new_retries = Column(Integer(), default=0)
+    update_retries = Column(Integer(), default=0)
+    max_new_retries = Column(Integer(), default=3)
+    max_update_retries = Column(Integer(), default=0)
+    new_poll_period = Column(Interval(), default=datetime.timedelta(seconds=1))
+    update_poll_period = Column(Interval(), default=datetime.timedelta(seconds=10))
+    errors = Column(JSONString(1024))
     _transform_metadata = Column('transform_metadata', JSON())
     _running_metadata = Column('running_metadata', JSON())
 
@@ -348,6 +365,7 @@ class Processing(BASE, ModelBase):
     workload_id = Column(Integer())
     status = Column(EnumWithValue(ProcessingStatus))
     substatus = Column(EnumWithValue(ProcessingStatus), default=0)
+    oldstatus = Column(EnumWithValue(ProcessingStatus), default=0)
     locking = Column(EnumWithValue(ProcessingLocking))
     submitter = Column(String(20))
     submitted_id = Column(Integer())
@@ -360,6 +378,13 @@ class Processing(BASE, ModelBase):
     submitted_at = Column("submitted_at", DateTime)
     finished_at = Column("finished_at", DateTime)
     expired_at = Column("expired_at", DateTime)
+    new_retries = Column(Integer(), default=0)
+    update_retries = Column(Integer(), default=0)
+    max_new_retries = Column(Integer(), default=3)
+    max_update_retries = Column(Integer(), default=0)
+    new_poll_period = Column(Interval(), default=datetime.timedelta(seconds=1))
+    update_poll_period = Column(Interval(), default=datetime.timedelta(seconds=10))
+    errors = Column(JSONString(1024))
     _processing_metadata = Column('processing_metadata', JSON())
     _running_metadata = Column('running_metadata', JSON())
     output_metadata = Column(JSON())
@@ -543,6 +568,7 @@ class Message(BASE, ModelBase):
     transform_id = Column(Integer())
     processing_id = Column(Integer())
     num_contents = Column(Integer())
+    retries = Column(Integer(), default=0)
     created_at = Column("created_at", DateTime, default=datetime.datetime.utcnow)
     updated_at = Column("updated_at", DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)
     msg_content = Column(JSON())
@@ -553,17 +579,46 @@ class Message(BASE, ModelBase):
                    Index('MESSAGES_TYPE_ST_PR_IDX', 'msg_type', 'status', 'destination', 'processing_id'))
 
 
+class Command(BASE, ModelBase):
+    """Represents the operations commands"""
+    __tablename__ = 'commands'
+    cmd_id = Column(BigInteger().with_variant(Integer, "sqlite"),
+                    Sequence('COMMAND_ID_SEQ', schema=DEFAULT_SCHEMA_NAME),
+                    primary_key=True)
+    request_id = Column(BigInteger().with_variant(Integer, "sqlite"))
+    workload_id = Column(Integer())
+    transform_id = Column(Integer())
+    processing_id = Column(Integer())
+    cmd_type = Column(EnumWithValue(CommandType))
+    status = Column(EnumWithValue(CommandStatus))
+    substatus = Column(Integer())
+    locking = Column(EnumWithValue(CommandLocking))
+    username = Column(String(20))
+    retries = Column(Integer(), default=0)
+    source = Column(EnumWithValue(CommandLocation))
+    destination = Column(EnumWithValue(CommandLocation))
+    created_at = Column("created_at", DateTime, default=datetime.datetime.utcnow)
+    updated_at = Column("updated_at", DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)
+    cmd_content = Column(JSON())
+    errors = Column(JSONString(1024))
+
+    _table_args = (PrimaryKeyConstraint('cmd_id', name='COMMANDS_PK'),
+                   Index('COMMANDS_TYPE_ST_IDX', 'cmd_type', 'status', 'destination', 'request_id'),
+                   Index('COMMANDS_TYPE_ST_TF_IDX', 'cmd_type', 'status', 'destination', 'transform_id'),
+                   Index('COMMANDS_TYPE_ST_PR_IDX', 'cmd_type', 'status', 'destination', 'processing_id'))
+
+
 def register_models(engine):
     """
     Creates database tables for all models with the given engine
     """
 
     # models = (Request, Workprogress, Transform, Workprogress2transform, Processing, Collection, Content, Health, Message)
-    models = (Request, Transform, Processing, Collection, Content, Health, Message)
+    models = (Request, Transform, Processing, Collection, Content, Health, Message, Command)
 
     for model in models:
-        if not engine.has_table(model.__tablename__, model.metadata.schema):
-            model.metadata.create_all(engine)   # pylint: disable=maybe-no-member
+        # if not engine.has_table(model.__tablename__, model.metadata.schema):
+        model.metadata.create_all(engine)   # pylint: disable=maybe-no-member
 
 
 def unregister_models(engine):
@@ -572,7 +627,7 @@ def unregister_models(engine):
     """
 
     # models = (Request, Workprogress, Transform, Workprogress2transform, Processing, Collection, Content, Health, Message)
-    models = (Request, Transform, Processing, Collection, Content, Health, Message)
+    models = (Request, Transform, Processing, Collection, Content, Health, Message, Command)
 
     for model in models:
         model.metadata.drop_all(engine)   # pylint: disable=maybe-no-member
