@@ -5,12 +5,12 @@ Test client.
 """
 
 import argparse
-import os
+# import os
 
-os.environ['PANDA_AUTH'] = 'oidc'
-os.environ['PANDA_URL_SSL'] = 'https://pandaserver-doma.cern.ch:25443/server/panda'
-os.environ['PANDA_URL'] = 'http://pandaserver-doma.cern.ch:25080/server/panda'
-os.environ['PANDA_AUTH_VO'] = 'Rubin'
+# os.environ['PANDA_AUTH'] = 'oidc'
+# os.environ['PANDA_URL_SSL'] = 'https://pandaserver-doma.cern.ch:25443/server/panda'
+# os.environ['PANDA_URL'] = 'http://pandaserver-doma.cern.ch:25080/server/panda'
+# os.environ['PANDA_AUTH_VO'] = 'Rubin'
 # os.environ['PANDA_CONFIG_ROOT'] = '~/.panda/'
 
 from idds.common.constants import RequestStatus, ProcessingStatus    # noqa E402
@@ -24,14 +24,13 @@ parser.add_argument('--task_id', dest='task_id', action='store', help='Task to k
 
 
 def kill_workflow_task(idds_server, request_id, task_id=None):
-    if task_id is None:
-        msg = {'command': 'update_request', 'parameters': {'status': RequestStatus.ToCancel}}
-    else:
-        msg = {'command': 'update_processing', 'parameters': [{'workload_id': task_id, 'status': ProcessingStatus.ToCancel}]}
-
     c = pandaclient.idds_api.get_api(idds_utils.json_dumps,
-                                     idds_host=idds_server, compress=True, manager=False)
-    ret = c.send_message(request_id=request_id, msg=msg)
+                                     idds_host=idds_server, compress=True, manager=True)
+    if task_id is None:
+        ret = c.abort(request_id=request_id)
+    else:
+        ret = c.abort_tasks(request_id=request_id, task_id=task_id)
+
     print("Command is sent to iDDS: ", str(ret))
 
 
